@@ -174,10 +174,14 @@ class TestListArchives(DBQueryTestCase):
                                          'btype ON archiveinfo.btype_id = btype.id',
                                          'win_archives ON archiveinfo.id = win_archives.archive_id'],
                                   clauses=['win_archives.relpath = %(relpath)s',
-                                           r"platforms ~ E'\\mall\\M'",
-                                           r"flags ~ E'\\mA\\M'",
-                                           r"flags ~ E'\\mB\\M'"],
-                                  values={'relpath': 'somerelpath'},
+                                           r"platforms ~ %(platforms_pattern_0)s",
+                                           r"flags ~ %(flags_pattern_0)s",
+                                           r"flags ~ %(flags_pattern_1)s"],
+                                  values={'relpath': 'somerelpath',
+                                          'flags_pattern_0': '\\mA\\M',
+                                          'flags_pattern_1': '\\mB\\M',
+                                          'platforms_pattern_0': '\\mall\\M',
+                                          },
                                   colsByAlias={'relpath': 'win_archives.relpath',
                                                'platforms': 'win_archives.platforms',
                                                'flags': 'win_archives.flags',
